
//+------------------------------------------------------------------+
//|                      OM NAMAH SIVAY                              |
//|        OM NAMAH SIVAY              OM NAMAH SIVAY                |
//|    OM NAMAH SIVAY    OM NAMAH SIVAY    OM NAMAH SIVAY            |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, OM Corp."
#property link "http://www.jatinpruthvi.co.cc"
//+------------------------------------------------------------------+
//| CLASS DECLARATION                                                |
//+------------------------------------------------------------------+
class MySymbolExpert
{
	//--- private members
private:
	//   double LOTS; //Lots or volume to Trade
	//  double TradePct; //Percentage of Account Free Margin to trade
	double ADX_min; // ADX Minimum value
	int ADX_handle; // ADX Handle
	int MA_handle; // Moving Average Handle
	double plus_DI[]; // array to hold ADX +DI values for each bars
	double minus_DI[]; // array to hold ADX -DI values for each bars
	double MA_val[]; // array to hold Moving Average values for each bars
	double ADX_val[]; // array to hold ADX values for each bars
	double Closeprice; // variable to hold the previous bar closed price

	string symbol; // variable to hold the current symbol name
	ENUM_TIMEFRAMES period; // variable to hold the current timeframe value
	string Errormsg; // variable to hold our error messages
	int Errcode; // variable to hold our error codes

	//--- Public member/functions
public:
	void MySymbolExpert(); // Class Constructor
	void setSymbol(string syb)
	{
		symbol = syb;
	} // function to set current symbol
	void setPeriod(ENUM_TIMEFRAMES prd)
	{
		period = prd;
	} // function to set current symbol timeframe/period


	void setadxmin(double adx)
	{
		ADX_min = adx;
	} // function to set ADX Minimum values
	void doInit(int adx_period, int ma_period); // function to be used at our EA intialization
	void doUninit(); // function to be used at EA de-initialization
	bool checkBuy(); // function to check for Buy conditions
	bool checkSell(); // function to check for Sell conditions
	ENUM_ORDER_TYPE getBuySellSignal();
	bool doMQLRates();

	//--- Protected members
protected:
	void showError(string msg, int ercode); // function for use to display error messages
	void getBuffers(); // function for getting Indicator buffers

}; // end of class declaration
//+------------------------------------------------------------------+
//| Definition of our Class/member functions                         |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|  This CLASS CONSTRUCTOR                                          |
//|  *Does not have any input parameters                             |
//|  *Initilizes all the necessary variables                         |
//+------------------------------------------------------------------+
void MySymbolExpert::MySymbolExpert()
{
	//---initialize all necessary variables

	ZeroMemory(ADX_val);
	ZeroMemory(MA_val);
	ZeroMemory(plus_DI);
	ZeroMemory(minus_DI);
	Errormsg = "";
	Errcode = 0;
}
//+-----------------------------------------------------------------------+
//| OUR PUBLIC FUNCTIONS                                                  |
//+-----------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| DOINIT FUNCTION                                                  |
//| *Takes the ADX indicator's Period and Moving Average indicator's |
//| period as input parameters                                       |
//| *To be used in the OnInit() function of our EA                   |
//+------------------------------------------------------------------+
void MySymbolExpert::doInit(int adx_period, int ma_period)
{
	//--- get handle for ADX indicator
	ADX_handle = iADX(symbol, period, adx_period);
	//--- get the handle for Moving Average indicator
	MA_handle = iMA(symbol, period, ma_period, 0, MODE_EMA, PRICE_CLOSE);
	//--- what if handle returns Invalid Handle
	if (ADX_handle < 0 || MA_handle < 0) {
		Errormsg = "Error Creating Handles for indicators";
		Errcode = GetLastError();
		showError(Errormsg, Errcode);
	}
	//--- set Arrays as series
	//--- the ADX values arrays
	ArraySetAsSeries(ADX_val, true);
	//--- the +DI value arrays
	ArraySetAsSeries(plus_DI, true);
	//--- the -DI value arrays
	ArraySetAsSeries(minus_DI, true);
	//--- the MA values arrays
	ArraySetAsSeries(MA_val, true);
}
//+------------------------------------------------------------------+
//|  SHOWERROR FUNCTION                                              |
//|  *Input Parameters - Error Message, Error Code                   |
//+------------------------------------------------------------------+
void MySymbolExpert::showError(string msg, int ercode)
{
	Print(msg, "-error:", ercode, "!!"); // display error
}

//+------------------------------------------------------------------+
//|  GETBUFFERS FUNCTION                                             |
//|  *No input parameters                                            |
//|  *Uses the class data members to get indicator's buffers         |
//+------------------------------------------------------------------+
void MySymbolExpert::getBuffers()
{
	if (CopyBuffer(ADX_handle, 0, 0, 3, ADX_val) < 0 || CopyBuffer(ADX_handle, 1, 0, 3, plus_DI) < 0 ||
		CopyBuffer(ADX_handle, 2, 0, 3, minus_DI) < 0 || CopyBuffer(MA_handle, 0, 0, 3, MA_val) < 0) {
		Errormsg = "Error copying indicator Buffers";
		Errcode = GetLastError();
		showError(Errormsg, Errcode);
	}
}


//+------------------------------------------------------------------+
//|  DOUNINIT FUNCTION                                               |
//|  *No input parameters                                            |
//|  *Used to release ADX and MA indicators handles                  |
//+------------------------------------------------------------------+
void MySymbolExpert::doUninit()
{
	//--- release our indicator handles
	IndicatorRelease(ADX_handle);
	IndicatorRelease(MA_handle);
}

bool MySymbolExpert::doMQLRates()
{
	MqlRates mrate[];
	ArraySetAsSeries(mrate, true);
	if (CopyRates(symbol, period, 0, 3, mrate) < 0) {
		Print("Error copying rates/history data - error:", GetLastError(), "!!");
		return false;
	}
	Closeprice = mrate[1].close;
	return true;
}
// 0 - nothing , 1-Buy , 2-Sell
ENUM_ORDER_TYPE MySymbolExpert::getBuySellSignal()
{
	ENUM_ORDER_TYPE nSignal = -1;
	int count = 0;
	if (checkBuy()) {
		nSignal = ORDER_TYPE_BUY;
		count++;
	}
	if (checkSell()) {
		nSignal = ORDER_TYPE_SELL;
		count++;
	}
	if (count != 0 && count < 2) {
		return nSignal;
	}
	else {
		return -1;
	}
}
//+------------------------------------------------------------------+
//| CHECKBUY FUNCTION                                                |
//| *No input parameters                                             |
//| *Uses the class data members to check for Buy setup              |
//| based on the defined trade strategy                              |
//| *Returns TRUE if Buy conditions are met or FALSE if not met      |
//+------------------------------------------------------------------+
bool MySymbolExpert::checkBuy()
{
	if (!doMQLRates()) {
		return false;
	}

	/*
	Check for a Long/Buy Setup : MA increasing upwards,
	previous price close above MA, ADX > ADX min, +DI > -DI
	*/
	getBuffers();
	//--- declare bool type variables to hold our Buy Conditions
	bool Buy_Condition_1 = (MA_val[0] > MA_val[1]) && (MA_val[1] > MA_val[2]); // MA Increasing upwards
	bool Buy_Condition_2 = (Closeprice > MA_val[1]);                           // previous price closed above MA
	bool Buy_Condition_3 = (ADX_val[0] > ADX_min);     // Current ADX value greater than minimum ADX value
	bool Buy_Condition_4 = (plus_DI[0] > minus_DI[0]); // +DI greater than -DI
	//--- Putting all together
	if (Buy_Condition_1 && Buy_Condition_2 && Buy_Condition_3 && Buy_Condition_4) {
		return (true);
	}
	else {
		return (false);
	}
}

//+------------------------------------------------------------------+
//| CHECKSELL FUNCTION                                               |
//| *No input parameters                                             |
//| *Uses the class data members to check for Sell setup             |
//|  based on the defined trade strategy                             |
//| *Returns TRUE if Sell conditions are met or FALSE if not met     |
//+------------------------------------------------------------------+
bool MySymbolExpert::checkSell()
{

	/*
	Check for a Short/Sell Setup : MA decreasing downwards,
	previous price close below MA, ADX > ADX min, -DI > +DI
	*/
	getBuffers();
	//--- declare bool type variables to hold our Sell Conditions
	bool Sell_Condition_1 = (MA_val[0] < MA_val[1]) && (MA_val[1] < MA_val[2]); // MA decreasing downwards
	bool Sell_Condition_2 = (Closeprice < MA_val[1]);                           // Previous price closed below MA
	bool Sell_Condition_3 = (ADX_val[0] > ADX_min);     // Current ADX value greater than minimum ADX
	bool Sell_Condition_4 = (plus_DI[0] < minus_DI[0]); // -DI greater than +DI

	//--- putting all together
	if (Sell_Condition_1 && Sell_Condition_2 && Sell_Condition_3 && Sell_Condition_4) {
		return (true);
	}
	else {
		return (false);
	}
}

//+------------------------------------------------------------------+
//| OPENBUY FUNCTION                                                 |
//| *Has Input parameters - order type, Current ASK price,           |
//|  Stop Loss, Take Profit, deviation, comment                      |
//| *Checks account free margin before pacing trade if trader chooses|
//| *Prints of a success if position is opened or shows error        |
//+------------------------------------------------------------------+
